#!/usr/bin/env python

# INFO: The setup-script for the screenlets

# (c) 2008 Natan Yellin (Aantn) <aantny@gmail.com>
# Original authors Helder Fraga (Whise) <helder.fraga@hotmail.com> 
#  and Rico Pfaus (RYX) <ryx@ryxperience.com> 
#

import os
import subprocess
import sys

from distutils.core import setup
import distutils.sysconfig

# import screenlets to get package constants
#sys.path.insert(0, 'src/lib/')
#import screenlets

PACKAGES = ['screenlets','screenlets.plugins']

# ---------------------------------------------------------------
# Create a list of extra non-library files to pass to setup()
# ---------------------------------------------------------------

# Scripts that will be installed into PREFIX/bin/
scripts_list = ['src/bin/melange',
	'src/bin/screenlets-daemon',
	'src/bin/universal-applets-manager',
	'src/bin/universal-applets-packager',
	'src/bin/screenlets-cli',
	'src/bin/ua-sidebar']

# The manager's files that will go into PREFIX/share/universal-applets-manager
files_list = [('share/universal-applets-manager', 
	['src/share/universal-applets-manager/noimage.svg',
	'src/share/universal-applets-manager/KarambaScreenlet.py',
	'src/share/universal-applets-manager/widget.png',
	'src/share/universal-applets-manager/WidgetScreenlet.py',
	'src/share/universal-applets-manager/prefs.js',
	'src/share/universal-applets-manager/webapp.png',
	'src/share/universal-applets-manager/WebappScreenlet.py',
	'src/share/universal-applets-manager/karamba.png'])]

# DBUS service files that should go into /usr/share/dbus-1/services
files_list.insert(0, ('/usr/share/dbus-1/services',
	['src/share/melange/org.UniversalApplets.Melange.service',
	'src/share/universal-applets-manager/org.UniversalApplets.UniversalAppletsManager.service',
	'src/share/screenlets-daemon/org.UniversalApplets.Daemon.service']))

# .desktop files and icons
files_list.insert(0, ('share/applications', ['desktop-menu/universal-applets-manager.desktop']))
files_list.insert(0, ('share/icons', ['desktop-menu/screenlets.svg']))

# GtkBuilder xml files
files_list.insert(0, ('share/melange', ['src/share/melange/gtk_builder_gui.xml']))

# Translation files
podir = os.path.join (os.path.realpath ("."), "po")
if os.path.isdir (podir):
	if subprocess.call(["which", "msgfmt"]) != 0:
		raise Exception("msgfmt could not be found. Please install gettext.")
	buildcmd = "msgfmt -o build/locale/%s/LC_MESSAGES/%s.mo po/%s.po"
	mopath = "build/locale/%s/LC_MESSAGES/%s.mo"
	destpath = "share/locale/%s/LC_MESSAGES"
	for name in os.listdir (podir):		
		if name.endswith('.po'):
			name = name.replace('universal-applets-manager','screenletsmanager')
			dname = name.split('-')[1].split('.')[0]
			name = name[:-3]
			name = name.replace('screenletsmanager','universal-applets-manager')
			if sys.argv[1] == "build" or sys.argv[1] == "install":
				print 'Creating language Binary for : ' + name
				if not os.path.isdir ("build/locale/%s/LC_MESSAGES" % dname):
					os.makedirs ("build/locale/%s/LC_MESSAGES" % dname)
				os.system (buildcmd % (dname,name.replace('-'+dname,''), name))
				files_list.append ((destpath % dname, [mopath % (dname,name.replace('-'+dname,''))]))

# ----------------------------
# Call setup()
# ----------------------------

setup(name='universal-applets',
	version			= '0.1',
	description		= 'Universal-Applets is a widget framework for Linux.',
	url				= 'http://www.theesylum.com',
	author			= '(c) Natan Yellin (Aantn) <aantny@gmail.com>',
	author_email	= 'aantny@gmail.com',
	license			= 'GPL v3',
	packages		= PACKAGES,
	package_dir		= {'screenlets': 'src/lib/universal-applets'},
	data_files		= files_list,
	scripts			= scripts_list
)
