# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license. 
# Thank you for using free software!

# Universal Applets Manager Module
#
# (c) 2008 Natan Yellin
#
# See AUTHORS for original authors

# Module containing functions and classes used by universal-applets-manager
# and screenlets-daemon.

import os
import gettext
import gtk
import dbus.service
import screenlets, utils
import shutil

gettext.textdomain('universal-applets')
gettext.bindtextdomain('universal-applets',
	screenlets.INSTALL_PREFIX +  '/share/locale')

_ = gettext.gettext

if os.geteuid() == 0:
	# we run as root, install system-wide
	USER = 0
	DIR_USER = screenlets.PATH
	DIR_AUTOSTART = '/etc/xdg/autostart' # TODO: use pyxdg here
else:
	# we run as normal user, install into $HOME
	USER = 1
	DIR_USER = screenlets.USER_PATH
	DIR_AUTOSTART = utils.get_autostart_dir()
	

class ScreenletInstaller:
	"""A simple utility to install screenlets into the current user's directory 
	(so either into $HOME/.screenlets/ for normal users or, if run as root, 
	into screenlets.INSTALL_PREFIX/share/screenlets/)."""
	
	def __init__ (self):
		self._message = ''
	
	def get_info_from_package_name (self, filename):
		"""Return all info we can get from the package-name or return None
		if something went wrong. If nothing failed, the returned value is
		a 4-tuple of the form: (name, version, basename, extension)."""
		base = os.path.basename(filename)
		ext = str(filename)[len(str(filename)) -3:]
		# prepend "tar." if we have a bz2 or gz archive
		tar_opts = 'xfz'
		if ext == 'bz2':
			tar_opts = 'xfj'
		if ext == 'skz': 
			screenlets.show_error(None, _('This type of karamba theme is ' + 
				'not supported yet\n Only older versions can be used.'))
			return False
		# extract archive to temporary dir
		
		if not os.path.isdir('/tmp/screenlets/'):
			os.mkdir('/tmp/screenlets/')
		
		shutil.rmtree('/tmp/screenlets/install-temp', True)
		
		os.mkdir('/tmp/screenlets/install-temp/')
		
		os.system('tar %s "%s" -C %s' % (tar_opts, filename, tmpdir))
		
		for dd in os.listdir(tmpdir):
			if str(dd).endswith('.theme'):
				os.system('mv ' + tmpdir + ' ' + '/tmp/screenlets/' + dd[:-6])
				os.system('mkdir %s' % tmpdir)
				os.system('mv ' + '/tmp/screenlets/' + dd[:-6] + ' '+ tmpdir)
				name = dd[:-6]
				return (name, ext)
		
		name = None
		for d in tmpdir : #for each item in folders
			if os.path.exists(d) and os.path.isdir(d): #is it a valid folder?
				for f in os.listdir(tmpdir): 
					name = f
		
		if name is not None:
			return (name, ext)
		else:
			return False
	
	def get_result_message (self):
		"""Return a human-readable result message about the last operation."""
		return self._message
	
	def is_installed (self, name):
		"""Checks if the given applet with the name defined by 'name' 
		(without trailing 'Screenlet') is already installed in the current
		install target location."""
		return os.path.isdir(DIR_USER + '/' + name)
	
	def install (self, filename):
		"""Install a applet from a given archive-file. Extracts the
		contents of the archive to the user's applets dir."""
		result = False
		# get name of screenlet
		#basename	= os.path.basename(filename)
		#extension	= os.path.splitext(filename)
		#name		= basename[:basename.find('.')]
		#print name
		info = self.get_info_from_package_name(filename)
		if info == False:
			self._message= _("Archive is damaged or unsupported, use only tar, bz2 or gz.")
			return False
		name = info[0]
		ext = info[1]
		
		tar_opts = 'xfz'
		if ext == 'bz2':
			tar_opts = 'xfj'
		
		# check if screenlet is already installed
		#found_path = screenlets.utils.find_first_screenlet_path(name)
		if self.is_installed(name):#found_path != None:
			if not screenlets.show_question(None,(_("The %sScreenlet is already installed in '%s'.\nDo you wish to continue?") % (name, DIR_USER)),(_('Install %s') % (name))):
				self._message= _('%sScreenlet is already installed') % (name)
				return False
		# check extension and create appropriate args for tar
		tmpdir = screenlets.TEMP_DIR + '/install-temp'
		# verify contents
		if not os.path.isdir('%s/%s' % (tmpdir, name)):
			# dir missing
			self._message = _("Invalid archive. Archive must contain a directory with the screenlet's name.")
		elif not os.path.isfile('%s/%s/%sScreenlet.py' % (tmpdir, name, name)):
			# Not a screenlet , lets check for karamba theme
			themename = ''
			for findtheme in os.listdir('%s/%s/' % (tmpdir, name)):
				if str(findtheme).lower().endswith('.theme'):
					if themename == '':
						if not os.path.isfile(tmpdir + '/'+ name + '/' + findtheme[:-6] + '.py'):
							themename = findtheme[:-6]
						else:
							self._message = _("Compatibility for this karamba theme is not yet implemented")
							return False
			if themename != '':
				name1 = name.replace(' ','')
				name1 = name1.replace('-','')
				name1 = name1.replace('.','')
				name1 = name1.replace('_','')
				#os.system('tar %s %s -C %s' % (tar_opts, chr(34)+filename+chr(34), DIR_USER))
				#os.system('mkdir %s/%s' % (DIR_USER,name1))
				if self.is_installed(name1):os.system('rm -rf %s/%s' % (DIR_USER, name1))
				
				os.system('mv %s/%s %s/%s ' % (chr(34) + tmpdir, name + chr(34),DIR_USER,name1))#tar %s %s -C %s' % (tar_opts, chr(34)+filename+chr(34), DIR_USER))
				os.system('mkdir %s/%s/themes' % (DIR_USER,name1))
				os.system('mkdir %s/%s/themes/default' % (DIR_USER,name1))
				os.system('cp ' + screenlets.INSTALL_PREFIX + '/share/universal-applets-manager/karamba.png ' + DIR_USER + '/' + name1 + '/themes/default/')
				if os.path.isfile(DIR_USER + '/' + name1 + '/icon.png') == False or os.path.isfile(DIR_USER + '/' + name1 + '/icon.svg') == False:
					os.system('cp ' + screenlets.INSTALL_PREFIX + '/share/universal-applets-manager/karamba.png ' + DIR_USER + '/' + name1 + '/icon.png')
				widgetengine = open(screenlets.INSTALL_PREFIX + '/share/universal-applets-manager/KarambaScreenlet.py', 'r')
				enginecopy = widgetengine.read()
				widgetengine.close()
				widgetengine = None
				enginecopy = enginecopy.replace('KarambaScreenlet',name1 + 'Screenlet')

				enginesave = open(DIR_USER + '/' + name1 + '/' + name1 + 'Screenlet.py','w')
				enginesave.write(enginecopy)
				enginesave.close()
				self._message = _("Karamba theme was successfully installed")
				result = True	
			else:self._message = _("Invalid archive. Archive does not contain a screenlet or a valid karamba theme.")
		else:
			# check for package-info

			if not os.path.isfile('%s/%s/Screenlet.package' % (tmpdir, name)):
				if screenlets.show_question(None,(_("%s was not packaged with the screenlet packager. Do you wish to continue and try to install it?") % (name)),(_('Install %s') % (name))):
					pass
				else:
					self._message = _("This package was not packaged with the screenlet packager.")
					return False	
			
			# copy archive to user dir (and create if not exists)
			utils.create_user_dir()
			os.system('tar %s %s -C %s' % (tar_opts, chr(34)+filename+chr(34), DIR_USER))
			# delete package info from target dir
			os.system('rm %s/%s/Screenlet.package' % (DIR_USER, name))
			# set msg/result
			self._message = _("The %sScreenlet has been succesfully installed.") % name
			result = True
		# remove temp contents
		os.system('rm -rf %s/install-temp' % screenlets.TEMP_DIR)
		# ready
		return result

class ScreenletsController:
	"""Class containing functions for controlling and installing screenlets."""
	
	def __init__ (self):
		"""Constructor"""
		# fallback icon for screenlets without an icon
		self.default_img = gtk.gdk.pixbuf_new_from_file_at_size(\
			screenlets.INSTALL_PREFIX + '/share/universal-applets-manager/noimage.svg', 
			56, 56)
		
		if not os.path.isdir(screenlets.USER_PATH):
			os.mkdir(screenlets.USER_PATH)
	
	#--------------------------------------------------
	# Functions to install files
	#--------------------------------------------------
	
	def install_screenlet (self, filename):
		"""Install a screenlet from a given archive-file. Extracts the
		contents of the archive to the user's screenlet dir."""
		installer = ScreenletInstaller()
		result = installer.install(filename)
		if result: 	
			screenlets.show_message(None, installer.get_result_message())
		else:
			screenlets.show_error(None, installer.get_result_message())
		return result
	
	#---------------------------------------------------
	# Functions to open other applications
	#---------------------------------------------------
	
	def launch_browser (self, widget, url):
		os.system("xdg-open %s &" % url)
	
	def launch_manager (self, widget):
		"""Launch or stop universal-applets-manager - if the manager is running it will be stopped, if is not running - will be launched. 
		This function is used by screenlets-daemon as left-button-click handler"""
		bus = dbus.SessionBus()
		if bus:
			try:
				# Get a list of all currently-owned names on the bus - we're looking for running ScreenletsManager
				list_of_names = bus.list_names()
				for a_name in list_of_names:
					if a_name.find("UniversalAppletsManager") != -1:
						uamanager_is_running = True
				path = '/org/UniversalApplets/UniversalAppletsManager'
				proxy_obj = bus.get_object('org.UniversalApplets.UniversalAppletsManager', path)

				if proxy_obj:
					daemon_iface = dbus.Interface(proxy_obj, 'org.UniversalApplets.UniversalAppletsManager')
					if uamanager_is_running:
						daemon_iface.quit()

			except Exception, ex:
				pass
	
	def launch_screenlet(self, widget, screenlet_name):
		if not screenlets.launch_screenlet(screenlet_name):
			screenlets.show_error(None, _('Failed to add %sScreenlet.') % screenlet_name)
	
	#--------------------------------------------------
	# Functions to show dialog boxes
	#--------------------------------------------------
	
	def show_about_dialog (self, widget=None):
		"""Create and show the about dialog."""
		dlg = gtk.AboutDialog()
		# set url
		gtk.about_dialog_set_url_hook(self.launch_browser)
		# add other info
		dlg.set_name(screenlets.APP_NAME)
		dlg.set_comments(_(screenlets.COMMENTS))
		dlg.set_version(screenlets.VERSION)
		dlg.set_copyright(screenlets.COPYRIGHT)
		dlg.set_authors(screenlets.AUTHORS)
		dlg.set_website(screenlets.WEBSITE)
		dlg.set_website_label(screenlets.WEBSITE)
		dlg.set_license(_('This application is released under the GNU General Public License v3 (or, at your option, any later version). You can find the full text of the license under http://www.gnu.org/licenses/gpl.txt. By using, editing and/or distributing this software you agree to the terms and conditions of this license. Thank you for using free software!'))
		dlg.set_wrap_license(True)
		dlg.set_documenters(screenlets.DOCUMENTERS)
		dlg.set_artists(screenlets.ARTISTS)
		dlg.set_translator_credits(screenlets.TRANSLATORS)
		# add logo
		dlg.set_logo(self.pixbuf)
		# run and then destroy
		dlg.run()
		dlg.destroy()
	
	def show_install_dialog (self, widget=None):		
		"""Create and show the install-dialog."""
		# create filter
		flt = gtk.FileFilter()
		for pattern in ['*.tar.bz2', '*.tar.gz', '*.tar', '*.zip']:
			flt.add_pattern(pattern)

		# create dialog
		dlg = gtk.FileChooserDialog(buttons=(gtk.STOCK_CANCEL, 
			gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		dlg.set_current_folder(os.environ['HOME'])
		dlg.set_title((_('Install a Screenlet')))
		dlg.set_filter(flt)
		
		# run dialog
		resp		= dlg.run()
		filename	= dlg.get_filename()
		dlg.destroy()
		
		if resp == gtk.RESPONSE_OK:
			# try installing and show result dialog
			return self.install_screenlet (filename)
		return False
	
	#--------------------------------------------------
	# Functions to quit running screenlets
	#--------------------------------------------------
	
	def quit_all_screenlets (self, widget):
		"""This function closes all running applets,. It uses unregister_screenlet and quit_screenlet_by_name functions 
		to close all applets that have been returned by utils.list_running_applets()"""
		running_applets = utils.list_running_applets()
		for an_applet in running_applets:
			if an_applet.endswith('Screenlet'):
				an_applet = an_applet[:-9]
			self.unregister_screenlet(an_applet)
			self.quit_screenlet_by_name(an_applet)
	
	def quit_screenlet_by_name (self, name):
		"""Quit all instances of the given screenlet type."""
		# get service for instance and call quit method
		service = screenlets.services.get_service_by_name(name)
		if service:
			service.quit()
