#!/usr/bin/env python
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license. 
# Thank you for using free softwa

# Flickr module by Helder Fraga aka whise <helder.fraga@hotmail.com>
# Improvements to photo matching by Peter Ward aka flowblok <peteraward@gmail.com>


from urllib import urlopen, basejoin
import re
import Proxy

imgtag = r'<img.*?src="(.*?)".*?>'
anchorwrap = r'<a.*?href="(.*?)".*?>.*?%s.*?</a>' % imgtag

container = re.compile(r'<span.*?class="photo_container pc_\w".*?>.*?%s.*?</span>' % anchorwrap, re.DOTALL)
photodiv = re.compile(r'()<div.*?class="photoImgDiv".*?>.*?%s.*?</div>' % imgtag, re.DOTALL)

def iter_add(iter1, iter2):
	for i in iter1:
		yield i
	for i in iter2:
		yield i

class Flickr:
	url_list = {}
	
	def get_image_list(self, url):
		"""Returns a Tuple containing images links and webpage links of a flickr url"""
		list_a = []
		self.url_list = {}
		proxies = Proxy.Proxy().get_proxy()	
		sourcetxt = urlopen(url,proxies=proxies).read()
		while sourcetxt.find("Photo" + chr(34)) != -1:
			image = sourcetxt[sourcetxt.find("Photo" + chr(34))+7:]
			sourcetxt = image
			sourceimage = image[image.find("a href=" + chr(34))+8:]
			sourceimage = sourceimage[:sourceimage.find(chr(34)) ].strip()
			realimage = image[image.find("mg src=" + chr(34))+8:]
			realimage = realimage[:realimage.find(chr(34)) ].strip()
			imageurl = 'http://www.flickr.com' + sourceimage
			list_a.append(realimage)
			self.url_list[realimage] = imageurl
		return list_a
	
	def fetch_images(self, baseurl, size='m'):
		"""Generates image links and webpage links of a Flickr url"""
		found = set()
		
		proxies=None
		if Proxy:
			proxies = Proxy.Proxy().get_proxy()
		
		# Generates URLs for Flickr pages.
		def make_urls(baseurl):
			yield baseurl
			
			sep = '?'
			if '?' in baseurl:
				sep = '&'
			
			# /explore urls have randomised images.
			if not '/explore' in baseurl:
				i = 2
				while True:
					yield '%s%spage=%d' % (baseurl, sep, i)
					i += 1
		
		for url in make_urls(baseurl):
			imgs = []
			sourcetxt = urlopen(url,proxies=proxies).read()
			
			containers = container.finditer(sourcetxt)
			photodivs = photodiv.finditer(sourcetxt)
			
			for match in iter_add(containers, photodivs):
				link, image = match.groups()
				if link:
				    link = basejoin(url, link)
				else:
				    link = url
				
				imgparts = image.split('.')
				imgparts = ('.'.join(imgparts[:-1]), imgparts[-1])
				if imgparts[0][-2] == '_':
					image = imgparts[0][:-1] + size + '.' + imgparts[1]
				
				imgid = imgparts[0]
				
				if '?' in link:
					linkid = '?'.join(link.split('?')[:-1])
				else:
					linkid = link
				
				if (imgid, linkid) not in found:
					found.add((imgid, linkid))
					imgs.append((image, link))
			
			if imgs:
				yield imgs
			else:
				break
	
	def save_image(self,url,path):
		"""Saves the image from url in path"""
		proxies = Proxy.Proxy().get_proxy()
		sourcetxt = urlopen(url,proxies=proxies).read()
		fileObj = open( path,"w") #// open for for write
		fileObj.write(sourcetxt)
		fileObj.close()
