# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license. 
# Thank you for using free software!

# Universal Applets/Screenlets DBUS module
# (c) 2008 Natan Yellin (Aantn) <aantny@gmail.com>

import sys

import dbus
import dbus.service
if getattr(dbus, 'version', (0,0,0)) <= (0,80,0):
	import dbus.glib
else:
	from dbus.mainloop.glib import DBusGMainLoop
	DBusGMainLoop(set_as_default=True)

import screenlets
import screenlets.logger

log = screenlets.logger.get_default_logger()

class SingletonApplication (dbus.service.Object):
	"""Base class for singleton applications and dbus services."""
	
	def __init__ (self, bus, path, iface):
		"""Quits the app if another instance of the application is running."""		
		log.info("Checking for another running instance...")
		try:
			# attempt to init the DBUS service (this will fail if the service is already running)
			super(SingletonApplication, self).__init__(dbus.service.BusName(bus, 
				bus=dbus.SessionBus(), do_not_queue=True), path)
			log.info("No other instances found.")
		except dbus.exceptions.NameExistsException, ex:
			# If the user ran the app with the --replace option then kill the old server and continue to run
			if screenlets.COMMAND_LINE_OPTIONS.SESSION_REPLACE == True:
				log.debug("Replacing currently running process.")
				proxy_obj = dbus.SessionBus().get_object(bus, path)
				if proxy_obj:
					iface = dbus.Interface(proxy_obj, iface)
					iface.quit()
					# reinitialize the DBUS service
					dbus.service.Object.__init__(self, dbus.service.BusName(bus, 
						bus=dbus.SessionBus(), do_not_queue=False), path)
			# if the server is already running and the --replace flag is NOT present then just output a message and quit
			else:
				log.critical("Instance found. Please use --replace to quit it and start a new instance.")
				sys.exit(1)
