# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license. 
# Thank you for using free software!

##@package utils.py
# universal-applets.utils.py by Universal-Applets Core Team
# See AUTHORS for original authors
#


import os
import re
import sys
import stat
import urllib

import dbus
import gtk
import gettext
import gobject
try:
	import gnomevfs
except:
	pass

import screenlets

gettext.textdomain('screenlets')
gettext.bindtextdomain('screenlets', screenlets.INSTALL_PREFIX +  '/share/locale')
_ = gettext.gettext

# ------------------------------------------------------------------------------
# FUNCTIONS
# ------------------------------------------------------------------------------


def get_autostart_dir():
	"""This function returns the system autostart directory. Works for gnome, KDE and xfce"""
	desktop_environment = 'gnome' #TODO: this function is too simple to use  temporary variables - we can shorten it

	if os.environ.get('KDE_FULL_SESSION') == 'true':
		desktop_environment = 'kde'
	elif os.environ.get('GNOME_DESKTOP_SESSION_ID'):
		desktop_environment = 'gnome'
	else:
		try:
			import commands
			info = commands.getoutput('xprop -root _DT_SAVE_MODE')
			if ' = "xfce4"' in info:
				desktop_environment = 'xfce'
		except (OSError, RuntimeError):
			pass

	if desktop_environment == 'kde':
		return os.environ['HOME'] + '/.kde/Autostart/'
	elif desktop_environment == 'gnome':
		return os.environ['HOME'] + '/.config/autostart/'
	elif desktop_environment == 'xfce':
		return os.environ['HOME'] + '/.config/autostart/'

if os.geteuid()==0:
	# we run as root, install system-wide
	USER = 0
	DIR_USER		= screenlets.INSTALL_PREFIX + '/share/screenlets'
	DIR_AUTOSTART	= '/etc/xdg/autostart'			# TODO: use pyxdg here
else:
	# we run as normal user, install into $HOME
	USER = 1
	DIR_USER		= os.environ['HOME'] + '/.screenlets'
	DIR_AUTOSTART = get_autostart_dir()


def containsAll(str, set):
	"""Check whether 'str' contains ALL of the chars in 'set'"""
	for c in set:
		if c not in str:
			return False
	return True

def containsAny(str, set):
	"""Check whether 'str' contains ANY of the chars in 'set'"""
	for c in set:
		if c in str:
			return True

def create_autostarter(name):
	"""Create a .desktop-file for the screenlet with the given name in 
	$HOME/.config/autostart."""
	if not os.path.isdir(DIR_AUTOSTART):
		os.system('mkdir %s' % DIR_AUTOSTART)
		if not os.path.isdir(DIR_AUTOSTART):
			screenlets.show_error(None, _("The autostart directory does not exist and could not be created.\
			 Please manually create the directory:\n%s") % DIR_AUTOSTART, _('Error'))
			return False
	if name.endswith('Screenlet'):
		name = name[:-9]
	starter = '%s%sScreenlet.desktop' % (DIR_AUTOSTART, name)

	for a_file in os.listdir(DIR_AUTOSTART):
		if a_file.find(name + 'Screenlet') == 0: #Check if applet nameis _exactly_ the same as name of autostart file
			print str(a_file) + ' duplicate entry'
			os.system('rm %s%s' % (chr(34)+DIR_AUTOSTART,f+chr(34)))
			print _('Removed duplicate entry')
			
	if not os.path.isfile(starter):
		path = find_first_screenlet_path(name)
		if path:
			print _("Create autostarter for: %s/%sScreenlet.py") % (path, name)
			code = ['[Desktop Entry]']
			code.append('Name=%sScreenlet' % name)
			code.append('Encoding=UTF-8')
			code.append('Version=1.0')
			code.append('Type=Application')
			code.append('Exec= python -u -t %s/%sScreenlet.py' % (path, name))
			code.append('X-GNOME-Autostart-enabled=true')
			#print code
			f = open(starter, 'w')
			if f:
				for l in code:
					f.write(l + '\n')
				f.close()
				return True
			print _('Failed to create autostarter for %s.') % name
			return False
	else:
		print _("Starter already exists.")
		return True

def delete_autostarter(name):
	"""Delete the autostart for the given screenlet."""
	if name.endswith('Screenlet'):
		name = name[:-9]
	print _('Delete autostarter for %s.') % name
	os.system('rm %s%sScreenlet.desktop' % (DIR_AUTOSTART, name))
	for f in os.listdir(DIR_AUTOSTART):
		a = f.find(name + 'Screenlet')
		if a != -1:
			print str(f) + ' duplicate entry'
			os.system('rm %s%s' % (chr(34)+DIR_AUTOSTART,f+chr(34)))
			print _('Removed duplicate entry')

def create_user_dir():
	"""Create the userdir for the screenlets."""
	if not os.path.isdir(os.environ['HOME'] + '/.screenlets'):
		try:
			os.mkdir(os.environ['HOME'] + '/.screenlets')
		except OSError:
			print "couldn't create user dir"


def find_first_screenlet_path(name):
	"""Scan the SCREENLETS_PATH for the first occurence of screenlet "name" and 
	return the full path to it. This function is used to get the theme/data 
	directories for a Screenlet."""
	for dir in screenlets.SCREENLETS_PATH:
		path = os.path.join(dir, name)
		if not os.path.isdir(path):
			continue
		name_py = name + 'Screenlet.py'
		name_py_abs = os.path.join(path, name_py)
		if os.path.isfile(name_py_abs):
			return path
	# nothing found
	return None

def get_screenlet_icon(screenlet_name, width, height):
	if width == -1 or height == -1:
		img = gtk.gdk.pixbuf_new_from_file(\
			screenlets.INSTALL_PREFIX + '/share/universal-applets-manager/noimage.svg')
	else:
		img = gtk.gdk.pixbuf_new_from_file_at_size(\
			screenlets.INSTALL_PREFIX + '/share/universal-applets-manager/noimage.svg', width, height)
	for path in screenlets.SCREENLETS_PATH:
		applet_path = os.path.join(path, screenlet_name)
		for ext in ['svg', 'png']:
			img_path = os.path.join(applet_path, "icon." + ext)
			if os.path.isfile(img_path):
				try:
					if width == -1 or height == -1:
						img = gtk.gdk.pixbuf_new_from_file(img_path)
					else:
						img = gtk.gdk.pixbuf_new_from_file_at_size(img_path, width, height)
				except Exception, ex:
					# TODO: Find specific exception thrown by pixbuf_new_from_file*.
					pass
	return img

def get_between(data, first, last):
	"""Function returns part of the 'data' string contained between 'first' and 'last' elements """
	begin = data.find(first) + len(first)
	end = data.find(last, begin)
	return data[begin:end]

def get_screenlet_metadata(screenlet_name):
	"""Returns a dict with name, info, author and version of the given
	screenlet. Use with care because it always imports the screenlet 
	module and shouldn't be used too often due to performance issues."""
	# find path to file
	path = find_first_screenlet_path(screenlet_name)
	classname = screenlet_name + 'Screenlet'

	try:
		slfile = open(path + '/'+ classname + '.py', 'r')
		sldata = slfile.read()
		slfile.close()
		name = get_between(sldata, '__name__', '\n')
		name1 = get_between(name, "'", "'")
		if name1.find(' = ') != -1:
			name1 = get_between(name, '"', '"')
		info = get_between(sldata, '__desc__', '\n')
		info1 = get_between(info, "'", "'")
		if info1.find(' = ') != -1:
			info1 = get_between(info, '"', '"')
		if info1.find('_doc_') != -1:
			info1 = get_between(sldata, 'class ' + classname, '__name__')		
			info1 = get_between(info1, '"""', '"""')
		author = get_between(sldata, '__author__', '\n')
		author1 = get_between(author, "'", "'")
		if author1.find(' = ') != -1:
			author1 = get_between(author, '"', '"')
		version = get_between(sldata, '__version__', '\n')
		version1 = get_between(version, "'", "'")
		if version1.find(' = ') != -1:
			version1 = get_between(version, '"', '"')
		return {'name' : name1,
			'info' : info1, 
			'author' : author1, 
			'version' : version1}
	except (IOError, Exception):
		try:
			# add path to PYTHONPATH
			if sys.path.count(path) == 0:
				sys.path.insert(0, path)
			slmod = __import__(classname)
			cls = getattr(slmod, classname)
			sys.path.remove(path)
			return {'name' : cls.__name__, 
				'info' : cls.__desc__, 
				'author' : cls.__author__, 
				'version' : cls.__version__}
		except Exception, ex:
			print _("Unable to load '%s' from %s: %s ") % (screenlet_name, path, ex)
			return None

def get_available_screenlets_in_path(path):
	"""Returns all screenlets in the given path.
	No checking of this path is done - it may or
	may not be in SCREENLETS_PATH."""
	if os.path.isdir(path):
		for applet in os.listdir(path):
			name_py = os.path.join(path, applet, applet + 'Screenlet.py')
			if os.path.isfile(name_py):
				yield applet

def list_available_screenlets():
	"""Scan the SCREENLETS_PATHs for all existing applets and return their
	names (without trailing "Screenlet") as a list of strings."""
	#define empty list of applets
	list_of_applets = []
	
	#search all directories in screenlets.SCREENLETS_PATH
	lst = list_available_screenlets_by_path()
	for applets in lst:
		list_of_applets += applets
	
	# Return unique names only.
	return list(set(list_of_applets))

def list_available_screenlets_by_path(path=None):
	"""Scan the SCREENLETS_PATHs for all existing screenlets and return their
	names (without trailing "Screenlet") as lists organized by the path.
	If path is not None, only the screenlets in the given path will be returned."""
	all = []
	if path is None:
		paths = screenlets.SCREENLETS_PATH
	else:
		paths = [path]
	
	for p in paths:
		lst = list(get_available_screenlets_in_path(p))
		lst.sort()
		all.append(lst)
	return all

def list_running_applets():
	"""Returns a list with names of running screenlets."""
	#define empty list - it will be filled with currently running applets
	list_of_applets = []
	#check if  screenlets.TEMP_FILE exists, it's the file with names of running applets
	if os.path.isfile(screenlets.TEMP_FILE):
		#open the file
		temp_file = open(screenlets.TEMP_FILE, 'r')
		#check if ithe file has been opened
		if temp_file:
			#read line by line with names of applet
			for an_applet in temp_file.readlines():
				#if the name after stripping trailing 'Screenlet' is not None...
				if get_screenlet_process(an_applet[:-10]) is not None:
					#...add it to list of running applets
					list_of_applets.append(an_applet[:-1])	# strip trailing EOL
			#close the file
			temp_file.close()
	return list_of_applets

def get_screenlet_process(name):
	"""Returns the PID of the given screenlet (if running) or None."""
	# the following command gets a list of all running tasks (ps aux),
	# finds all of the relevant lines (grep), filters out the command
	# we're now running (grep -v), and uses awk to pick out the PID
	p = os.popen("ps aux | grep %sScreenlet.py | grep -v grep | awk '{print $2}'"
		% name)
	line = p.readlines()
	p.close()
	if len(line) > 0:
		return int(line[0][:-1])
	return None

def get_user_dir(key, default):
	"""http://www.freedesktop.org/wiki/Software/xdg-user-dirs"""

	user_dirs_dirs = os.path.expanduser("~/.config/user-dirs.dirs")
	if os.path.exists(user_dirs_dirs):
		f = open(user_dirs_dirs, "r")
		for line in f.readlines():
			if line.startswith(key):
				return os.path.expandvars(line[len(key)+2:-2])
	return default	

def get_desktop_dir():
	"""Returns desktop dir"""
	desktop_dir =  get_user_dir("XDG_DESKTOP_DIR", os.path.expanduser("~/Desktop"))
	desktop_dir = urllib.unquote(desktop_dir)
	return desktop_dir

def get_filename_on_drop(sel_data):
	"""Returns filenames of window droped files"""
	filename = ''
	filenames = []
	# get text-elements in selection data
	try:
		txt = unicode.encode(sel_data.get_text(), 'utf-8')
	except:
		txt = sel_data.get_text()
	txta = urllib.unquote(txt)
	txta = str(txta).split('\n')
		
	for txt in txta:
		if txt and txt != '':
			# if it is a filename, use it
			if txt.startswith('file://'):
				filename = txt[7:]
			else:
				print 'Invalid string: %s.' % txt
		else:
			# else get uri-part of selection
			uris = sel_data.get_uris()
			if uris and len(uris)>0:
				#print "URIS: "+str(uris	)
				filename = uris[0][7:]
		if filename != '':
			filenames.append(chr(34) +filename + chr(34))

	return filenames

def LoadPlaces():
	"""Returns mount points in media"""
	mountlist = os.popen('mount -l').read()
	prog = re.compile("^/dev/.*?\son\s/media/(.*?) .*?(\[(.*?)\])?$", re.MULTILINE)
	return prog.findall(mountlist)

def LoadBookmarks():
	"""Returns gtk bookmarks """
	_bookmarks_path = os.path.expanduser("~/.gtk-bookmarks")
	_places = []
	try:
		for line in file(_bookmarks_path):
			line = line.strip()

			if " " in line:
				uri, name = line.split(" ", 1)
				
			else:
				uri = line
				
				path = urllib.splittype(uri)[1]
				name = urllib.unquote(os.path.split(path)[1])

			try:
				if os.path.exists(uri):
					continue
		# Protect against a broken bookmarks file
			except TypeError:
				continue

			_places.append((uri, name))
		return _places
	except IOError, err:
		print "Error loading GTK bookmarks:", err

def quit_screenlet_by_name( name):
	"""Quit all instances of the given screenlet type."""
	# get service for instance and call quit method
	service = screenlets.services.get_service_by_name(name)
	if service:
		service.quit()

def quit_all_screenlets():
	a = list_running_applets()
	if a is not None:
		for s in a:
			if s.endswith('Screenlet'):
				s = s[:-9]
			try:
				quit_screenlet_by_name(s)
			except:
				pass

def restart_all_screenlets():
	quit_all_screenlets()
	for s in os.listdir(DIR_AUTOSTART):
		if s.lower().endswith('screenlet.desktop'):
			#s = s[:-17]
			os.system('sh '+ DIR_AUTOSTART + s + ' &')

def readMountFile( filename):
	"""Reads fstab file"""
	fstab = []
	f = open(filename, 'r')
	for line in f:
		if (not line.isspace() and not line.startswith('#') and not line.lower().startswith('none')) :
			fstabline = line.split()
			if fstabline[1] != 'none' and fstabline[1] != '/proc': fstab.append(fstabline[1])
	
	fstab.sort()
	return fstab

def read_file( filename):
	"""Reads a file"""
	f = open(filename, 'r')
	t = f.read()
	f.close()
	return t


def strip_html(string):
	"""Strips HTML tags of a string"""
	return re.sub(r"<.*?>|</.*?>","",string)

def lookup_daemon_autostart():
	"""Adds Screenlets-daemon to autostart if not already"""
	if not os.path.isdir(DIR_AUTOSTART):
	# create autostart directory, if not existent
		if screenlets.show_question(None, _("There is no existing autostart directory for your user account yet. Do you want me to automatically create it for you?"), _('Error')):
			print _("Auto-create autostart dir ...")
			os.system('mkdir %s' % DIR_AUTOSTART)
			if not os.path.isdir(DIR_AUTOSTART):
				screenlets.show_error(None, _("Automatic creation failed. Please manually create the directory:\n%s") % DIR_AUTOSTART, _('Error'))
				return False
		else:
			screenlets.show_message(None, _("Please manually create the directory:\n%s") % DIR_AUTOSTART)
			return False
	starter = '%sScreenlets Daemon.desktop' % (DIR_AUTOSTART)

	if not os.path.isfile(starter) and os.path.isfile('%sscreenlets-daemon.desktop' % (DIR_AUTOSTART)) == False:
		print _("Create autostarter for: Screenlets Daemon")
		code = ['[Desktop Entry]']
		code.append('Encoding=UTF-8')
		code.append('Version=1.0')
		code.append('Name=Screenlets Daemon')
		code.append('Type=Application')
		code.append('Exec=screenlets-daemon')
		code.append('X-GNOME-Autostart-enabled=true')
		f = open(starter, 'w')
		if f:
			for l in code:
				f.write(l + '\n')
			f.close()
			return True
		print _('Failed to create autostarter for %s.') % name
		return False
	else:
		print _("Starter already exists.")
		return True

def launch_screenlet(screenlet):
	"""Launches a screenlet"""
	name = str(screenlet)
	if not screenlets.launch_screenlet(name):
		screenlets.show_error(None, _('Failed to add %sScreenlet.') % name)

def xdg_open(name):
	"""Opens anything"""
	os.system('xdg-open ' + name + ' &')

# ------------------------------------------------------------------------------
# CLASSES
# ------------------------------------------------------------------------------

class ScreenletInfo:
	"""A container with info about a screenlet."""
	def __init__(self, name, lname, info, author, version, icon):
		self.name = name
		self.lname = lname
		self.info = info.replace("\n", '').replace('\t', ' ')
		self.author = author
		self.version = version
		self.icon = icon
		self.active = False
		self.system = not os.path.isfile('%s/%s/%sScreenlet.py' % (DIR_USER, name, name))
		self.autostart = os.path.isfile(DIR_AUTOSTART + '/' + name + 'Screenlet.desktop')

class FileMonitor(gobject.GObject):
	"""A simple wrapper around Gnome VFS file monitors.  Emits created, deleted,
	and changed events.  Incoming events are queued, with the latest event
	cancelling prior undelivered events."""
	
	
	__gsignals__ = {
		"event" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
			(gobject.TYPE_STRING, gobject.TYPE_INT)),
		"created" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
		"deleted" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
		"changed" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,))
	}

	def __init__(self, path):
		gobject.GObject.__init__(self)
		
		if os.path.isabs(path):
			self.path = "file://" + path
		else:
			self.path = path
		try:
			self.type = gnomevfs.get_file_info(path).type
		except gnomevfs.Error:
			self.type = gnomevfs.MONITOR_FILE

		self.monitor = None
		self.pending_timeouts = {}

	def open(self):
		if not self.monitor:
			if self.type == gnomevfs.FILE_TYPE_DIRECTORY:
				monitor_type = gnomevfs.MONITOR_DIRECTORY
			else:
				monitor_type = gnomevfs.MONITOR_FILE
			self.monitor = gnomevfs.monitor_add(self.path, monitor_type, self._queue_event)

	def _clear_timeout(self, info_uri):
		try:
			gobject.source_remove(self.pending_timeouts[info_uri])
			del self.pending_timeouts[info_uri]
		except KeyError:
			pass

	def _queue_event(self, monitor_uri, info_uri, event):
		self._clear_timeout(info_uri)
		self.pending_timeouts[info_uri] = \
			gobject.timeout_add(250, self._timeout_cb, monitor_uri, info_uri, event)

	def queue_changed(self, info_uri):
		self._queue_event(self.path, info_uri, gnomevfs.MONITOR_EVENT_CHANGED)

	def close(self):
		gnomevfs.monitor_cancel(self.monitor)
		self.monitor = None

	def _timeout_cb(self, monitor_uri, info_uri, event):
		if event in (gnomevfs.MONITOR_EVENT_METADATA_CHANGED,
			gnomevfs.MONITOR_EVENT_CHANGED):
			self.emit("changed", info_uri)
		elif event == gnomevfs.MONITOR_EVENT_CREATED:
			self.emit("created", info_uri)
		elif event == gnomevfs.MONITOR_EVENT_DELETED:
			self.emit("deleted", info_uri)
		self.emit("event", info_uri, event)

		self._clear_timeout(info_uri)
		return False


class IniReader:
	"""A simple config/ini-reader class. This is only used for reading the 
	theme.conf files yet, thus it only uses string-values.
	TODO: add writing-functions and let backend use this, too"""
	
	def __init__ (self):
		self.options	= []
		self.sections	= {}
	
	def list_options (self, section=''):
		"""Return all options (alternatively only from the given section)."""
		if section != '':
			return self.sections[section]
		else:
			return self.options
	
	def get_option (self, name, section=''):
		"""Get a variable from the config (optional: only get vars from the 
		specified section)."""
		if section != '':
			l = self.sections[section]
		else:
			l = self.options
		for i in l:
			if i[0] == name:
				return i[1]
		return None
	
	def has_section (self, name):
		"""Returns true if the given section exists."""
		return self.sections.has_key(name)
	
	def load_conf (self, filename):
		"""Load a config/ini-file and save vars in internal list."""
		f=None
		try:
			f = open (filename, "r")
		except:
			print _("File %s not found") % str(filename)
			
		if f:
			section_name = ''
			for line in f.readlines():
				# strip whitespace/tabs on the left
				line = line.lstrip().lstrip('\t')
				#print line
				# ignore comment, EOL and too short lines
				if len(line) < 4 or line[0] in ("#", "\n", ";"):
					pass
				else:
					# split var/value and trim
					tmp = line.split('=', 1)
					# no '=' found? check for section name
					if len(tmp) < 2 and len(line) > 5 and line[0] == '[':
						section_name = line[:-1][1:-1]
						self.sections[section_name] =  []
						#print "Section found: %s" % section_name
					else:
						# two entries? split var/value
						var = tmp[0].rstrip().rstrip('\t')
						val = tmp[1][:-1].lstrip()	# remove EOL
						#print "VAR: %s=%s" % (var, val)
						# and add them to lists
						if var != '' and val != '':
							o = [var, val]
							self.options.append(o)
							if section_name != '':
								try:
									self.sections[section_name].append(o)
								except:
									print _("Section %s not found!") % section_name
			f.close()
			return True
		else:
			return False
	def write_conf(self, buttons, filename):
		"""Write a config/ini-file"""

		out = ''
		for name, button in buttons.items():
			out += "%s=%s\n" % (name, button.get_active())
		# write the file
		f = open(filename, 'w')
		f.write("[Options]\n" + out)
		f.close()

class Notifier:
	"""A simple and conveniet wrapper for the notification-service. Allows
	screenlets to easily pop up notes with their own icon (if any)."""
	
	def __init__ (self, screenlet=None):
		self.bus = dbus.SessionBus()
		self.notifications = dbus.Interface(\
			self.bus.get_object('org.freedesktop.Notifications', 
			'/org/freedesktop/Notifications'), 'org.freedesktop.Notifications')
		self.screenlet = screenlet

	def notify (self, message, title='', icon='', timeout=-1, screenlet=None):
		"""Send a notification to org.freedesktop.Notifications. The message
		should contain the text you want to display, title may define a title
		(summary) for the message, icon can be the full path to an icon,
		timeout can be set to the desired displaying time in milliseconds."""
		if self.bus and self.notifications:
			if not screenlet:
				screenlet = self.screenlet
			if screenlet:
				p = find_first_screenlet_path(screenlet.__class__.__name__[:-9])
				if p:
					icon = p + '/icon.svg'
				title = screenlet.__name__
			self.notifications.Notify('Screenlets', 0, icon, title, message, 
				[], {}, timeout)
			return True
		else:
			print _("Notify: No DBus running or notifications-daemon unavailable.")
		return False
