#!/usr/bin/env python

import os
from screenlets.options import *
	
class TestObject (EditableOptions):
	
	testlist = ['test1', 'test2', 3, 5, 'Noch ein Test']
	pop3_account = ('Username', '')
	pin_x		= 100
	pin_y		= 6
	text_x		= 19
	text_y		= 35
	font_name	= 'Sans 12'
	rgba_color	= (0.0, 0.0, 1.0, 1.0)
	text_prefix	= '<b>'
	text_suffix	= '</b>'
	note_text	= ""	# hidden option because val has its own editing-dialog
	random_pin_pos	= True
	opt1 = 'testval 1'
	opt2 = 'testval 2'
	filename2	= ''
	filename	= ''
	dirname		= ''
	font = 'Sans 12'
	color = (0.1, 0.5, 0.9, 0.9)
	name = 'a name'
	name2 = 'another name'
	combo_test = 'el2'
	flt = 0.5
	x = 10
	y = 25
	width = 30
	height = 50
	is_sticky = False
	is_widget = False
	time	= (12, 32, 49)		# a time-value (tuple with ints)

	def __init__ (self):
		EditableOptions.__init__(self)
		# Add group
		self.add_options_group('General', 
			'The general options for this Object ...')
		self.add_options_group('Window', 
			'The Window-related options for this Object ...')
		self.add_options_group('Test', 'A Test-group ...')
		# Add editable options
		self.add_option(ListOption('Test', 'testlist', self.testlist,
			'ListOption-Test', 'Testing a ListOption-type ...'))
		self.add_option(StringOption('Window', 'name', 'TESTNAME',
			'Testname', 'The name/id of this Screenlet-instance ...'), 
			realtime=False)
		self.add_option(AccountOption('Test', 'pop3_account', 
			self.pop3_account, 'Username/Password', 
			'Enter username/password here ...'))
		self.add_option(StringOption('Window', 'name2', 'TESTNAME2',
			'String2', 'Another string-test ...'))
		self.add_option(StringOption('Test', 'combo_test', "el1", 'Combo', 
			'A StringOption displaying a drop-down-list with choices...', 
			choices=['el1', 'el2', 'element 3']))
		self.add_option(FloatOption('General', 'flt', 30, 
			'A Float', 'Testing a FLOAT-type ...', 
			min=0, max=gtk.gdk.screen_width(), increment=0.01, digits=4))
		self.add_option(IntOption('General', 'x', 30, 
			'X-Position', 'The X-position of this Screenlet ...', 
			min=0, max=gtk.gdk.screen_width()))
		self.add_option(IntOption('General', 'y', 30, 
			'Y-Position', 'The Y-position of this Screenlet ...',
			min=0, max=gtk.gdk.screen_height()))
		self.add_option(IntOption('Test', 'width', 300, 
			'Width', 'The width of this Screenlet ...', min=100, max=1000))
		self.add_option(IntOption('Test', 'height', 150, 
			'Height', 'The height of this Screenlet ...', 
			min=100, max=1000))
		self.add_option(BoolOption('General', 'is_sticky', True, 
			'Stick to Desktop', 'Show this Screenlet always ...'))
		self.add_option(BoolOption('General', 'is_widget', False,  
			'Treat as Widget', 'Treat this Screenlet as a "Widget" ...'))
		self.add_option(FontOption('Test', 'font', 'Sans 14', 
			'Font', 'The font for whatever ...'))
		self.add_option(ColorOption('Test', 'color', (1, 0.35, 0.35, 0.7), 
			'Color', 'The color for whatever ...'))
		self.add_option(FileOption('Test', 'filename', os.environ['HOME'],
			'Filename-Test', 'Testing a FileOption-type ...',
			patterns=['*.py', '*.pyc']))
		self.add_option(ImageOption('Test', 'filename2', os.environ['HOME'],
			'Image-Test', 'Testing the ImageOption-type ...'))
		self.add_option(DirectoryOption('Test', 'dirname', os.environ['HOME'],
			'Directory-Test', 'Testing a FileOption-type ...'))
		self.add_option(TimeOption('Test','time', self.time,
			'TimeOption-Test', 'Testing a TimeOption-type ...'))
		
		self.disable_option('width')
		self.disable_option('height')
		# TEST: load options from file
		#self.add_options_from_file('/home/ryx/Desktop/python/screenlets/screenlets-0.0.9/src/share/screenlets/Notes/options.xml')
	
	def __setattr__(self, name, value):
		self.__dict__[name] = value
		print name + "=" + str(value)
	
	def get_short_name(self):
		return self.__class__.__name__[:-6]			
		
class TestChildObject (TestObject):
	
	uses_theme = True
	theme_name = 'test'
	
	def __init__ (self):
		TestObject.__init__(self)
		self.add_option(StringOption('Test', 'anothertest', 'ksjhsjgd',
			'Another Test', 'An attribute in the subclass  ...'))
		self.add_option(StringOption('Test', 'theme_name', self.theme_name,
			'Theme', 'The theme for this Screenelt  ...',
			choices=['test1', 'test2', 'mytheme', 'blue', 'test']))


if __name__ == "__main__":	
	test = TestChildObject()
	se = OptionsDialog(500, 380)
	img = gtk.Image()
	img.set_from_file('../share/screenlets/Notes/icon.svg')
	se.set_info("EditableOptions Test Application",	"<span color='red'> Description goes here</span>",
		"(c) RYX 2007", version="v0.0.1", icon=img)
	se.show_options_for_object(test)
	resp = se.run()
	if resp == gtk.RESPONSE_OK:
		print "OK"
	else:
		print "Cancelled."
	se.destroy()
	print test.export_options_as_list()
