#!/usr/bin/env python

import gtk, gobject, cairo, screenlets, math
from screenlets.theme import Theme
from screenlets.drawing import Drawing

class Test (object):
	def __init__ (self):
		"""Constructor"""
		# create the main window
		self.window = gtk.Window()
		self.window.resize(500,500)
		self.window.set_app_paintable(True)
		screenlets.alpha_screen_changed(self.window)
		
		# connect events
		self.window.set_events(gtk.gdk.ALL_EVENTS_MASK)
		self.window.connect("expose-event", self.expose)
		self.window.connect("destroy", lambda w: gtk.main_quit())
		
		# set the theme and theme name
		self.theme = None
		self.theme_name = "test2"
		
		# load the theme
		self.change_theme()
		gobject.timeout_add(1500, self.change_theme)
		
		# show the window
		self.window.show_all()
	
	def change_theme (self):
		# free the old theme
		if self.theme is not None:
			self.theme.free()
			
		# decide the name of the new theme
		if self.theme_name == "test":
			self.theme_name = "test2"
		else:
			self.theme_name = "test"
			
		# load the new theme
		self.theme = Theme("./example-themes/%s" % self.theme_name)
		
		# redraw the window
		self.redraw_window()
		
		return True
	
	def expose (self, widget, event):
		self.width = widget.allocation.width
		self.height = widget.allocation.height
		
		ctx = widget.window.cairo_create()
		Drawing.clear_cairo_context(ctx)
		ctx.save()
		if self.theme is not None:
			self.theme.render(ctx, "grass")
		ctx.restore()
		
	def redraw_window (self):
		"""Redraw the entire Screenlet's window area."""
		x, y, w, h = self.window.get_allocation()
		rect = gtk.gdk.Rectangle(x, y, w, h)
		if self.window.window:
			self.window.window.invalidate_rect(rect, True)
			self.window.window.process_updates(True)
		
if __name__ == "__main__":
	app = Test()
	gtk.main()
